// apachemod/anyterm.h
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott

// This is version $Name: V1-1-4 $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


// This file defines common data types used for communication between
// the front and back-end processes, and entry points from the web
// server interface.

#ifndef anyterm_h
#define anyterm_h


#include <sys/ipc.h>
#include <sys/shm.h>


extern "C" {
#include <rote/rote.h>
};


#include "fifo.h"
#include "shm_alloc.h"
#include "lock.h"
#include "shm_buf.h"

#include <string>
#include <utility>


// Fixed character set:
#define CHARSET "iso-8859-1"

// Timeout for Anyterm sessions, seconds
#define ANYTERM_TIMEOUT 30

// Timeout for Wapterm sessions, seconds
#define WAPTERM_TIMEOUT 300

// Time after which all trace of a timed-out session is expunged
#define EXPUNGE_TIMEOUT 30


uint64_t hexstr_to_uint64(std::string s);
std::string uint64_to_hexstr(uint64_t i);


// Structure to store the state of an Anyterm session:

struct anyterm_backend;  // private backend state

class Session: public Shm::Obj {
 public:
  class id_t {
  public:
    id_t(void) {}
    id_t(uint64_t i): n(i) {}
    id_t(std::string s): n(hexstr_to_uint64(s)) {}
    std::string str(void) const { return uint64_to_hexstr(n); }
    bool operator==(id_t rhs) const { return n==rhs.n; }
  private:
    uint64_t n;
  };

  const id_t id;
  const int rows;
  const int cols;
  const int time_out;
  const Shm::CharBuf command;

  volatile bool open_req;  // request handler changes this
  volatile bool is_open;   // backend ROTE process changes this in response
  volatile Fifo key_fifo;
  /*volatile*/ Shm::Buf<RoteCell> screen;
  volatile int cur_row;
  volatile int cur_col;
  volatile bool dirty;
  /*volatile*/ Shm::CharBuf error_msg;
  volatile bool error;
  Shm::CharBuf old_html_screen;
  volatile time_t last_access;
  struct anyterm_backend* be;

  Shm::offset<Session> next;
  Shm::offset<Session> prev;

  Session(int r, int c, int t, std::string cmd);

  ReadWriteLock screen_lock;
  void touch(void);
  void report_any_backend_error(void);
  std::string send(std::string k);
  std::string rcv(void);
  std::string close(void);

};


class anyterm_globals: public Shm::Obj {
public:
  volatile pid_t backend_pid;
  ReadWriteLock session_list_lock;
  Shm::offset<Session> sessions;

  anyterm_globals(): sessions(NULL) {}

    void add_session(Session* s);
    void remove_session(Session* s);
};



// Find a session given its id

Session* get_session(Session::id_t s);


// Open a session (called by wapterm)

Session* session_open(std::string command, int rows, int cols,
			      int time_out);

std::string decode_uri_escapes(std::string s);

void signal_backend(void);

void touch_session (Session* ses);

void report_any_backend_error(Session* ses);



// Entry points from the web server interface:

void setup_backend_shmem(void);

typedef std::pair<std::string,std::string> response_t;

response_t process_request(std::string args, std::string command);


// Shared memory stuff:

extern key_t anyterm_shm_key;
extern int anyterm_shm_id;


#endif

