// apachemod/fifo.cc
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott

// This is version $Name: V1-1-4 $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include "fifo.h"

Fifo::Fifo(void):
  ipos(0), opos(0) {}


bool Fifo::empty(void) volatile
{
  return ipos == opos;
}


bool Fifo::full(void) volatile
{
  return (ipos+1)%sizeof(data) == opos;
}

int Fifo::push(const char* d, int n) volatile
{
  int p=0;
  while (!full() && p<n) {
    data[ipos]=d[p];
    ipos=(ipos+1)%sizeof(data);
    p++;
  }
  return p;
}

int Fifo::pop(char* d, int n) volatile
{
  int p=0;
  while (!empty() && p<n) {
    d[p]=data[opos];
    opos=(opos+1)%sizeof(data);
    p++;
  }
  return p;
}
