// apachemod/html.cc
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott

// This is version $Name: V1-1-4 $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include "html.h"

#include <string>

using namespace std;


// ROTE to HTML conversion:

static const char* colours[] = {"black", "red", "green", "yellow",
			 "blue", "magenta", "cyan", "white"};

static const unsigned char default_attr = 0x70;

static bool gen_style(string& h, unsigned char attr)
{
  if (attr!=default_attr) {
    h += string("<span style=\"")
      + (ROTE_ATTR_BOLD(attr)?"font-weight:bold;":"")
      + "color:" + colours[ROTE_ATTR_FG(attr)]
      + ";background-color:" + colours[ROTE_ATTR_BG(attr)]
      + ";\">";
    return true;
  }
  return false;
}

static const char* attr_end    = "</span>";

static const char* cursor_start = "<span class=\"cursor\">";
static const char* cursor_end   = "</span>";


string htmlify_screen(Session* ses)
{
  // Convert ROTE screen into HTML.
  // Slightly optimised to reduce spaces at right end of lines.

  string h;

  for (int r=0; r<ses->rows; r++) {
    int sp=0;
    bool styled=false;
    unsigned char prev_attr=0;
    for (int c=0; c<ses->cols; c++) {
      bool cursor = (r==ses->cur_row && c==ses->cur_col);
      RoteCell cell = ses->screen.data()[r*ses->cols+c];
      char ch = cell.ch;
      unsigned char at = cell.attr;
      string s;
      switch (ch) {
      case '<': s="&lt;"; break;
      case '>': s="&gt;"; break;
      case '&': s="&amp;"; break;
      case ' ': s="&#160;"; break;
      case '\\': s="\\\\"; break;
      default: s=ch; break;
      }

      if (ch==' ' && at==default_attr && !styled && c>0 && r>0 && !cursor) {
        sp++;
      } else {
	while (sp>0) {
	  h+="&#160;";
	  sp--;
	}
	if (styled && at!=prev_attr) {
	  h+=attr_end;
	}
	if (c==0 || at!=prev_attr) {
	  styled = gen_style(h,at);
	  prev_attr=at;
	}
	if (cursor) {
	  h+=cursor_start;
	}
	h+=s;
	if (cursor) {
	  h+=cursor_end;
	}
      }
    }
    if (styled) {
      h+=attr_end;
    }
    h+="\\n";
  }

  return h;
}



string wmlify_screen(Session* ses)
{
  // Convert ROTE screen into WML.
  // Slightly optimised to reduce spaces at right end of lines.

  string h;

  for (int r=0; r<ses->rows; r++) {
    int sp=0;
    for (int c=0; c<ses->cols; c++) {
      bool cursor = (r==ses->cur_row && c==ses->cur_col);
      RoteCell cell = ses->screen.data()[r*ses->cols+c];
      char ch = cell.ch;
      string s;
      switch (ch) {
      case '<': s="&lt;"; break;
      case '>': s="&gt;"; break;
      case '&': s="&amp;"; break;
      case '$': s="$$"; break;
      case ' ': s="&#160;"; break;
      default: s=ch; break;
      }

      if (ch==' ' && !cursor) {
        sp++;
      } else {
	while (sp>0) {
	  h+="&#160;";
	  sp--;
	}
	if (cursor) {
	  h+="<u>";
	}
	h+=s;
	if (cursor) {
	  h+="</u>";
	}
      }
    }
    h+="<br/>";
  }

  return h;
}
