// apachemod/lock.h
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott

// This is version $Name: V1-1-4 $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


// This file declares locking primitives.


#ifndef lock_h
#define lock_h

#include <unistd.h>

#include <apr_atomic.h>
#define atomic_type apr_uint32_t
#define atomic_add(var,val) apr_atomic_add(&(var),(atomic_type)(val))
#define atomic_compare_and_set(var,cmpwith,setto) (apr_atomic_cas(&(var),(setto),(cmpwith))==(cmpwith))


class Mutex {
 private:
  atomic_type locked;
 public:
  Mutex(): locked(false) {}
  friend class Lock;
};


class Lock {
 private:
  Mutex& m;
 public:
  Lock(Mutex& mu): m(mu) {
    while (1) {
/*       atomic_type old_locked = */
/* 	apr_atomic_cas(&m.locked, (atomic_type)true, (atomic_type)false); */
/*       if (!old_locked) { */
/* 	break; */
/*       } */
      if (atomic_compare_and_set(m.locked,(atomic_type)false,(atomic_type)true)) {
	break;
      }
      usleep(100);
    }
  }
  
    ~Lock() {
      m.locked=false;
    }
    
};



class ReadWriteLock {
 private:
  atomic_type lock;
  void spin(void) {
    usleep(100);
  }
  void writer_lock(void) {
    while (!atomic_compare_and_set(lock,0,1)) spin();
  }
  void writer_unlock(void) {
    atomic_add(lock,-1);
  }
  void reader_lock(void) {
    atomic_add(lock,2);
    while (lock&1) spin();
  }
  void reader_unlock(void) {
    atomic_add(lock,-2);
  }
 public:
  ReadWriteLock(void): lock(0) {}
    friend class Reader;
    friend class Writer;
};

class Writer {
 private:
  ReadWriteLock& lock;
 public:
  Writer(ReadWriteLock& l): lock(l) {
    lock.writer_lock();
  }
    ~Writer() {
      lock.writer_unlock();
    }
};

class Reader {
 private:
  ReadWriteLock& lock;
 public:
  Reader(ReadWriteLock& l): lock(l) {
    lock.reader_lock();
  }
    ~Reader() {
      lock.reader_unlock();
    }
};



#endif
