// apachemod/shm_buf.h
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005 Philip Endecott

// This is version $Name: V1-1-4 $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


// This file declares a string / character-buffer like type for shared
// memory.


#ifndef shm_buf_h
#define shm_buf_h


#include "shm_alloc.h"


namespace Shm {

  template <typename T>
  class Buf: public Obj {
  protected:
    static allocator<T> alloc;
    size_t l;
    offset<T> d;
    void setsize(size_t sz) {
      l=sz;
      if (l) {
	d=alloc.allocate(l);
      } else {
	d=NULL;
      }
    }
    void unset(void) {
      if (l) {
	alloc.deallocate(d,l);
      }
    }

  public:
    Buf() {
      setsize(0);
    }
    Buf(size_t sz) {
      setsize(sz);
    }
    ~Buf() {
      unset();
    }
    void resize(size_t sz) {
      unset();
      setsize(sz);
    }
    const T* data(void) const {
      return d;
    }
    T* data(void) {
      return d;
    }
    size_t length(void) const {
      return l;
    }
  };


  class CharBuf: public Buf<char> {
  private:
    void set_cptr(const char* cptr) {
      setsize(strlen(cptr)+1);
      strcpy(d,cptr);
    }
    void set_string(std::string s) {
      setsize(s.length()+1);
      strcpy(d,s.c_str());
    }

  public:
    CharBuf() {
      operator=("");
    }
    CharBuf(const char* cptr) {
      operator=(cptr);
    }
    CharBuf(std::string s) {
      operator=(s);
    }
    CharBuf& operator=(const char* cptr) {
      unset();
      set_cptr(cptr);
      return *this;
    }
    CharBuf& operator=(std::string s) {
      unset();
      set_string(s);
      return *this;
    }
    operator std::string() const {
      return std::string(d,l-1);
    }
    const char* data(void) const {
      return d;
    }
    size_t length(void) const {
      return l-1;
    }
    const char* c_str(void) const {
      return d;
    }
  };

};


#endif
